@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -38,6 +38,8 @@ static GNode *element_root = NULL;
 static GSList *driver_list = NULL;
 static gchar *device_filter = NULL;
 
+static gboolean started = FALSE;
+
 static struct {
 	enum connman_property_id id;
 	int type;
@@ -56,12 +58,6 @@ static struct {
 		DBUS_TYPE_STRING, "IPv4.Broadcast" },
 	{ CONNMAN_PROPERTY_ID_IPV4_NAMESERVER,
 		DBUS_TYPE_STRING, "IPv4.Nameserver" },
-
-	{ CONNMAN_PROPERTY_ID_WIFI_SECURITY,
-		DBUS_TYPE_STRING, "WiFi.Security" },
-	{ CONNMAN_PROPERTY_ID_WIFI_PASSPHRASE,
-		DBUS_TYPE_STRING, "WiFi.Passphrase" },
-
 	{ }
 };
 
@@ -104,6 +100,8 @@ static const char *type2string(enum connman_element_type type)
 		return "network";
 	case CONNMAN_ELEMENT_TYPE_SERVICE:
 		return "service";
+	case CONNMAN_ELEMENT_TYPE_PPP:
+		return "ppp";
 	case CONNMAN_ELEMENT_TYPE_IPV4:
 		return "ipv4";
 	case CONNMAN_ELEMENT_TYPE_IPV6:
@@ -116,63 +114,13 @@ static const char *type2string(enum connman_element_type type)
 		return "zeroconf";
 	case CONNMAN_ELEMENT_TYPE_CONNECTION:
 		return "connection";
+	case CONNMAN_ELEMENT_TYPE_VENDOR:
+		return "vendor";
 	}
 
 	return NULL;
 }
 
-static const char *subtype2string(enum connman_element_subtype type)
-{
-	switch (type) {
-	case CONNMAN_ELEMENT_SUBTYPE_UNKNOWN:
-		return "unknown";
-	case CONNMAN_ELEMENT_SUBTYPE_FAKE:
-		return "fake";
-	case CONNMAN_ELEMENT_SUBTYPE_NETWORK:
-		return "network";
-	case CONNMAN_ELEMENT_SUBTYPE_ETHERNET:
-		return "ethernet";
-	case CONNMAN_ELEMENT_SUBTYPE_WIFI:
-		return "wifi";
-	case CONNMAN_ELEMENT_SUBTYPE_WIMAX:
-		return "wimax";
-	case CONNMAN_ELEMENT_SUBTYPE_MODEM:
-		return "modem";
-	case CONNMAN_ELEMENT_SUBTYPE_BLUETOOTH:
-		return "bluetooth";
-	}
-
-	return NULL;
-}
-
-const char *__connman_element_policy2string(enum connman_element_policy policy)
-{
-	switch (policy) {
-	case CONNMAN_ELEMENT_POLICY_UNKNOWN:
-		return "unknown";
-	case CONNMAN_ELEMENT_POLICY_IGNORE:
-		return "ignore";
-	case CONNMAN_ELEMENT_POLICY_AUTO:
-		return "auto";
-	case CONNMAN_ELEMENT_POLICY_ASK:
-		return "ask";
-	}
-
-	return NULL;
-}
-
-enum connman_element_policy __connman_element_string2policy(const char *policy)
-{
-	if (strcasecmp(policy, "ignore") == 0)
-		return CONNMAN_ELEMENT_POLICY_IGNORE;
-	else if (strcasecmp(policy, "auto") == 0)
-		return CONNMAN_ELEMENT_POLICY_AUTO;
-	else if (strcasecmp(policy, "ask") == 0)
-		return CONNMAN_ELEMENT_POLICY_ASK;
-	else
-		return CONNMAN_ELEMENT_POLICY_UNKNOWN;
-}
-
 const char *__connman_ipv4_method2string(enum connman_ipv4_method method)
 {
 	switch (method) {
@@ -201,6 +149,7 @@ enum connman_ipv4_method __connman_ipv4_string2method(const char *method)
 		return CONNMAN_IPV4_METHOD_UNKNOWN;
 }
 
+#if 0
 static void append_property(DBusMessageIter *dict,
 				struct connman_property *property)
 {
@@ -308,485 +257,76 @@ static void set_common_property(struct connman_element *element,
 
 	__connman_element_unlock(element);
 }
+#endif
 
-static void emit_enabled_signal(DBusConnection *conn,
+static void emit_element_signal(DBusConnection *conn, const char *member,
 					struct connman_element *element)
 {
 	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *iface, *key;
 
-	DBG("conn %p", conn);
+	if (__connman_debug_enabled() == FALSE)
+		return;
+
+	DBG("conn %p member %s", conn, member);
 
 	if (element == NULL)
 		return;
 
-	switch (element->type) {
-	case CONNMAN_ELEMENT_TYPE_DEVICE:
-		iface = CONNMAN_DEVICE_INTERFACE;
-		key = "Powered";
-		break;
-	case CONNMAN_ELEMENT_TYPE_NETWORK:
-		iface = CONNMAN_NETWORK_INTERFACE;
-		key = "Connected";
-		break;
-	case CONNMAN_ELEMENT_TYPE_CONNECTION:
-		iface = CONNMAN_CONNECTION_INTERFACE;
-		key = "Default";
-		break;
-	default:
-		return;
-	}
-
 	signal = dbus_message_new_signal(element->path,
-						iface, "PropertyChanged");
+					CONNMAN_DEBUG_INTERFACE, member);
 	if (signal == NULL)
 		return;
 
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
-							&element->enabled);
-	dbus_message_iter_close_container(&entry, &value);
-
 	g_dbus_send_message(conn, signal);
 }
 
-static DBusMessage *do_update(DBusConnection *conn,
-					DBusMessage *msg, void *data)
+struct foreach_data {
+	enum connman_element_type type;
+	element_cb_t callback;
+	gpointer user_data;
+};
+
+static gboolean foreach_callback(GNode *node, gpointer user_data)
 {
-	struct connman_element *element = data;
+	struct connman_element *element = node->data;
+	struct foreach_data *data = user_data;
 
-	DBG("conn %p", conn);
+	DBG("element %p name %s", element, element->name);
 
-	if (element->enabled == FALSE)
-		return __connman_error_failed(msg);
+	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
+		return FALSE;
 
-	if (element->driver && element->driver->update) {
-		DBG("Calling update callback");
-		if (element->driver->update(element) < 0)
-			return __connman_error_failed(msg);
+	if (data->type != CONNMAN_ELEMENT_TYPE_UNKNOWN &&
+					data->type != element->type)
+		return FALSE;
 
-	}
+	if (data->callback)
+		data->callback(element, data->user_data);
 
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+	return FALSE;
 }
 
-static DBusMessage *do_enable(DBusConnection *conn,
-					DBusMessage *msg, void *data)
+void __connman_element_foreach(struct connman_element *element,
+				enum connman_element_type type,
+				element_cb_t callback, gpointer user_data)
 {
-	struct connman_element *element = data;
+	struct foreach_data data = { type, callback, user_data };
+	GNode *node;
 
-	DBG("conn %p", conn);
+	DBG("");
 
-	if (element->enabled == TRUE)
-		return __connman_error_failed(msg);
-
-	if (element->driver && element->driver->enable) {
-		DBG("Calling enable callback");
-		if (element->driver->enable(element) < 0)
-			return __connman_error_failed(msg);
-	}
-
-	element->enabled = TRUE;
-
-	emit_enabled_signal(connection, element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *do_disable(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-
-	DBG("conn %p", conn);
-
-	if (element->enabled == FALSE)
-		return __connman_error_failed(msg);
-
-	if (element->driver && element->driver->disable) {
-		DBG("Calling disable callback");
-		if (element->driver->disable(element) < 0)
-			return __connman_error_failed(msg);
-	}
-
-	element->enabled = FALSE;
-
-	emit_enabled_signal(connection, element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static void append_networks(struct connman_element *element,
-						DBusMessageIter *entry)
-{
-	DBusMessageIter value, iter;
-	const char *key = "Networks";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-
-	__connman_element_list(element, CONNMAN_ELEMENT_TYPE_NETWORK, &iter);
-
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
-}
-
-static DBusMessage *device_get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict, entry;
-	const char *str;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
-
-	str = subtype2string(element->subtype);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Type",
-						DBUS_TYPE_STRING, &str);
-
-	str = __connman_element_policy2string(element->policy);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Policy",
-						DBUS_TYPE_STRING, &str);
-
-	connman_dbus_dict_append_variant(&dict, "Powered",
-					DBUS_TYPE_BOOLEAN, &element->enabled);
-
-	if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIFI ||
-			element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIMAX) {
-		dbus_message_iter_open_container(&dict, DBUS_TYPE_DICT_ENTRY,
-								NULL, &entry);
-		append_networks(element, &entry);
-		dbus_message_iter_close_container(&dict, &entry);
-	}
-
-	add_common_properties(element, &dict);
-
-	dbus_message_iter_close_container(&array, &dict);
-
-	return reply;
-}
-
-static DBusMessage *device_set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessageIter iter, value;
-	const char *name;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	if (__connman_security_check_privileges(msg) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (g_str_equal(name, "Powered") == TRUE) {
-		dbus_bool_t powered;
-
-		dbus_message_iter_get_basic(&value, &powered);
-
-		if (powered == TRUE)
-			do_enable(conn, msg, element);
-		else
-			do_disable(conn, msg, element);
+	if (element != NULL) {
+		node = g_node_find(element_root, G_PRE_ORDER,
+						G_TRAVERSE_ALL, element);
+		if (node == NULL)
+			return;
 	} else
-		set_common_property(element, name, &value);
+		node = element_root;
 
-	__connman_element_store(element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+						foreach_callback, &data);
 }
 
-static int parse_network_dict(DBusMessageIter *iter, const char **ssid,
-				const char **security, const char **passphrase)
-{
-	while (dbus_message_iter_get_arg_type(iter) == DBUS_TYPE_DICT_ENTRY) {
-		DBusMessageIter entry, value;
-		const char *key;
-
-		dbus_message_iter_recurse(iter, &entry);
-		dbus_message_iter_get_basic(&entry, &key);
-
-		dbus_message_iter_next(&entry);
-		dbus_message_iter_recurse(&entry, &value);
-
-		switch (dbus_message_iter_get_arg_type(&value)) {
-		case DBUS_TYPE_STRING:
-			if (g_str_equal(key, "WiFi.SSID") == TRUE)
-				dbus_message_iter_get_basic(&value, ssid);
-			else if (g_str_equal(key, "WiFi.Security") == TRUE)
-				dbus_message_iter_get_basic(&value, security);
-			else if (g_str_equal(key, "WiFi.Passphrase") == TRUE)
-				dbus_message_iter_get_basic(&value, passphrase);
-			break;
-		}
-
-		dbus_message_iter_next(iter);
-	}
-
-	return 0;
-}
-
-static DBusMessage *device_create_network(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	struct connman_element *network;
-	DBusMessageIter iter, array;
-	const char *ssid = NULL, *security = NULL, *passphrase = NULL;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_recurse(&iter, &array);
-	parse_network_dict(&array, &ssid, &security, &passphrase);
-	if (ssid == NULL)
-		return __connman_error_invalid_arguments(msg);
-
-	DBG("ssid %s security %s passphrase %s", ssid, security, passphrase);
-
-	network = connman_element_create(ssid);
-
-	network->type = CONNMAN_ELEMENT_TYPE_NETWORK;
-	network->index = element->index;
-
-	network->remember = TRUE;
-
-	connman_element_add_static_property(network, "Name",
-						DBUS_TYPE_STRING, &ssid);
-
-	connman_element_add_static_array_property(element, "WiFi.SSID",
-					DBUS_TYPE_BYTE, &ssid, strlen(ssid));
-
-	network->wifi.security = g_strdup(security);
-	network->wifi.passphrase = g_strdup(passphrase);
-
-	connman_element_register(network, element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_OBJECT_PATH, &network->path,
-							DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *device_remove_network(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *network_get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-	const char *str;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
-
-	str = __connman_element_policy2string(element->policy);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Policy",
-						DBUS_TYPE_STRING, &str);
-
-	connman_dbus_dict_append_variant(&dict, "Available",
-					DBUS_TYPE_BOOLEAN, &element->available);
-
-	connman_dbus_dict_append_variant(&dict, "Connected",
-					DBUS_TYPE_BOOLEAN, &element->enabled);
-
-	connman_dbus_dict_append_variant(&dict, "Remember",
-					DBUS_TYPE_BOOLEAN, &element->remember);
-
-	add_common_properties(element, &dict);
-
-	dbus_message_iter_close_container(&array, &dict);
-
-	return reply;
-}
-
-static DBusMessage *network_set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessageIter iter;
-	DBusMessageIter value;
-	const char *name;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	if (__connman_security_check_privileges(msg) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (g_str_equal(name, "Remember") == TRUE) {
-		dbus_message_iter_get_basic(&value, &element->remember);
-	} else if (g_str_equal(name, "WiFi.Passphrase") == TRUE) {
-		const char *str;
-
-		dbus_message_iter_get_basic(&value, &str);
-		g_free(element->wifi.passphrase);
-		element->wifi.passphrase = g_strdup(str);
-	} else
-		set_common_property(element, name, &value);
-
-	__connman_element_store(element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *connection_get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-	const char *str;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
-
-	str = subtype2string(element->subtype);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Type",
-						DBUS_TYPE_STRING, &str);
-
-	if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIFI ||
-			element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIMAX)
-		connman_dbus_dict_append_variant(&dict, "Strength",
-					DBUS_TYPE_BYTE, &element->strength);
-
-	connman_dbus_dict_append_variant(&dict, "Default",
-					DBUS_TYPE_BOOLEAN, &element->enabled);
-
-	add_common_properties(element, &dict);
-
-	dbus_message_iter_close_container(&array, &dict);
-
-	return reply;
-}
-
-static DBusMessage *connection_set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessageIter iter, value;
-	const char *name;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	if (__connman_security_check_privileges(msg) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (g_str_equal(name, "Default") == TRUE) {
-		dbus_bool_t enabled;
-
-		dbus_message_iter_get_basic(&value, &enabled);
-
-		if (enabled == TRUE)
-			return do_enable(conn, msg, element);
-		else
-			return do_disable(conn, msg, element);
-	}
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static GDBusMethodTable device_methods[] = {
-	{ "GetProperties", "",      "a{sv}", device_get_properties },
-	{ "SetProperty",   "sv",    "",      device_set_property   },
-	{ "CreateNetwork", "a{sv}", "o",     device_create_network },
-	{ "RemoveNetwork", "o",     "",      device_remove_network },
-	{ "ProposeScan",   "",      "",      do_update             },
-	{ },
-};
-
-static GDBusMethodTable network_methods[] = {
-	{ "GetProperties", "",   "a{sv}", network_get_properties },
-	{ "SetProperty",   "sv", "",      network_set_property   },
-	{ "Connect",       "",   "",      do_enable              },
-	{ "Disconnect",    "",   "",      do_disable             },
-	{ },
-};
-
-static GDBusMethodTable connection_methods[] = {
-	{ "GetProperties", "",   "a{sv}", connection_get_properties },
-	{ "SetProperty",   "sv", "",      connection_set_property   },
-	{ },
-};
-
-static GDBusSignalTable element_signals[] = {
-	{ "PropertyChanged", "sv" },
-	{ },
-};
-
 struct append_filter {
 	enum connman_element_type type;
 	DBusMessageIter *iter;
@@ -807,7 +347,11 @@ static gboolean append_path(GNode *node, gpointer user_data)
 		return FALSE;
 
 	if (filter->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
-			element->subtype == CONNMAN_ELEMENT_SUBTYPE_NETWORK)
+			__connman_device_has_driver(element->device) == FALSE)
+		return FALSE;
+
+	if (filter->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
+			__connman_network_has_driver(element->network) == FALSE)
 		return FALSE;
 
 	dbus_message_iter_append_basic(filter->iter,
@@ -897,33 +441,13 @@ static gboolean match_driver(struct connman_element *element,
 	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
 		return FALSE;
 
-	if (element->type != driver->type &&
-			driver->type != CONNMAN_ELEMENT_TYPE_UNKNOWN)
-		return FALSE;
-
-	if (element->subtype == driver->subtype ||
-			driver->subtype == CONNMAN_ELEMENT_SUBTYPE_UNKNOWN)
+	if (element->type == driver->type ||
+			driver->type == CONNMAN_ELEMENT_TYPE_UNKNOWN)
 		return TRUE;
 
 	return FALSE;
 }
 
-static void enable_element(struct connman_element *element)
-{
-	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
-		return;
-
-	if (element->policy != CONNMAN_ELEMENT_POLICY_AUTO)
-		return;
-
-	if (element->driver && element->driver->enable) {
-		if (element->driver->enable(element) == 0) {
-			element->enabled = TRUE;
-			emit_enabled_signal(connection, element);
-		}
-	}
-}
-
 static gboolean probe_driver(GNode *node, gpointer data)
 {
 	struct connman_element *element = node->data;
@@ -938,8 +462,6 @@ static gboolean probe_driver(GNode *node, gpointer data)
 		__connman_element_lock(element);
 		element->driver = driver;
 		__connman_element_unlock(element);
-
-		enable_element(element);
 	}
 
 	return FALSE;
@@ -978,6 +500,9 @@ int connman_driver_register(struct connman_driver *driver)
 	driver_list = g_slist_insert_sorted(driver_list, driver,
 							compare_priority);
 
+	if (started == FALSE)
+		return 0;
+
 	if (element_root != NULL)
 		g_node_traverse(element_root, G_PRE_ORDER,
 				G_TRAVERSE_ALL, -1, probe_driver, driver);
@@ -985,22 +510,6 @@ int connman_driver_register(struct connman_driver *driver)
 	return 0;
 }
 
-static void disable_element(struct connman_element *element)
-{
-	if (element->policy != CONNMAN_ELEMENT_POLICY_AUTO)
-		return;
-
-	if (element->enabled == FALSE)
-		return;
-
-	if (element->driver && element->driver->disable) {
-		if (element->driver->disable(element) == 0) {
-			element->enabled = FALSE;
-			emit_enabled_signal(connection, element);
-		}
-	}
-}
-
 static gboolean remove_driver(GNode *node, gpointer data)
 {
 	struct connman_element *element = node->data;
@@ -1009,8 +518,6 @@ static gboolean remove_driver(GNode *node, gpointer data)
 	DBG("element %p name %s", element, element->name);
 
 	if (element->driver == driver) {
-		disable_element(element);
-
 		if (driver->remove)
 			driver->remove(element);
 
@@ -1062,9 +569,6 @@ struct connman_element *connman_element_create(const char *name)
 
 	element->name    = g_strdup(name);
 	element->type    = CONNMAN_ELEMENT_TYPE_UNKNOWN;
-	element->subtype = CONNMAN_ELEMENT_SUBTYPE_UNKNOWN;
-	element->state   = CONNMAN_ELEMENT_STATE_CLOSED;
-	element->policy  = CONNMAN_ELEMENT_POLICY_AUTO;
 	element->index   = -1;
 	element->enabled = FALSE;
 
@@ -1112,6 +616,8 @@ void connman_element_unref(struct connman_element *element)
 				g_atomic_int_get(&element->refcount) - 1);
 
 	if (g_atomic_int_dec_and_test(&element->refcount) == TRUE) {
+		if (element->destruct)
+			element->destruct(element);
 		free_properties(element);
 		g_free(element->ipv4.address);
 		g_free(element->ipv4.netmask);
@@ -1165,59 +671,6 @@ int connman_element_add_static_property(struct connman_element *element,
 	return 0;
 }
 
-static void emit_property_changed(DBusConnection *conn,
-				struct connman_element *element,
-				const char *name, int type, const void *data)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *iface, *sig;
-
-	DBG("conn %p", conn);
-
-	switch (element->type) {
-	case CONNMAN_ELEMENT_TYPE_DEVICE:
-		iface = CONNMAN_DEVICE_INTERFACE;
-		break;
-	case CONNMAN_ELEMENT_TYPE_NETWORK:
-		iface = CONNMAN_NETWORK_INTERFACE;
-		break;
-	case CONNMAN_ELEMENT_TYPE_CONNECTION:
-		iface = CONNMAN_CONNECTION_INTERFACE;
-		break;
-	default:
-		return;
-	}
-
-	signal = dbus_message_new_signal(element->path,
-						iface, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &name);
-
-	switch (type) {
-	case DBUS_TYPE_STRING:
-		sig = DBUS_TYPE_STRING_AS_STRING;
-		break;
-	case DBUS_TYPE_BYTE:
-		sig = DBUS_TYPE_BYTE_AS_STRING;
-		break;
-	default:
-		sig = DBUS_TYPE_VARIANT_AS_STRING;
-		break;
-	}
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-							sig, &value);
-	dbus_message_iter_append_basic(&value, type, data);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(conn, signal);
-}
-
 int connman_element_set_static_property(struct connman_element *element,
 				const char *name, int type, const void *value)
 {
@@ -1256,8 +709,6 @@ int connman_element_set_static_property(struct connman_element *element,
 
 	__connman_element_unlock(element);
 
-	emit_property_changed(connection, element, name, type, value);
-
 	return 0;
 }
 
@@ -1491,18 +942,6 @@ int connman_element_set_property(struct connman_element *element,
 		element->ipv4.nameserver = g_strdup(*((const char **) value));
 		__connman_element_unlock(element);
 		break;
-	case CONNMAN_PROPERTY_ID_WIFI_SECURITY:
-		__connman_element_lock(element);
-		g_free(element->wifi.security);
-		element->wifi.security = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
-	case CONNMAN_PROPERTY_ID_WIFI_PASSPHRASE:
-		__connman_element_lock(element);
-		g_free(element->wifi.passphrase);
-		element->wifi.passphrase = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
 	default:
 		return -EINVAL;
 	}
@@ -1517,6 +956,14 @@ int connman_element_get_value(struct connman_element *element,
 		return -EINVAL;
 
 	switch (id) {
+	case CONNMAN_PROPERTY_ID_IPV4_METHOD:
+		if (element->ipv4.method == CONNMAN_IPV4_METHOD_UNKNOWN)
+			return connman_element_get_value(element->parent,
+								id, value);
+		__connman_element_lock(element);
+		*((const char **) value) = __connman_ipv4_method2string(element->ipv4.method);
+		__connman_element_unlock(element);
+		break;
 	case CONNMAN_PROPERTY_ID_IPV4_ADDRESS:
 		if (element->ipv4.address == NULL)
 			return connman_element_get_value(element->parent,
@@ -1557,22 +1004,6 @@ int connman_element_get_value(struct connman_element *element,
 		*((char **) value) = element->ipv4.nameserver;
 		__connman_element_unlock(element);
 		break;
-	case CONNMAN_PROPERTY_ID_WIFI_SECURITY:
-		if (element->wifi.security == NULL)
-			return connman_element_get_value(element->parent,
-								id, value);
-		__connman_element_lock(element);
-		*((char **) value) = element->wifi.security;
-		__connman_element_unlock(element);
-		break;
-	case CONNMAN_PROPERTY_ID_WIFI_PASSPHRASE:
-		if (element->wifi.passphrase == NULL)
-			return connman_element_get_value(element->parent,
-								id, value);
-		__connman_element_lock(element);
-		*((char **) value) = element->wifi.passphrase;
-		__connman_element_unlock(element);
-		break;
 	default:
 		return -EINVAL;
 	}
@@ -1602,6 +1033,10 @@ gboolean connman_element_get_static_property(struct connman_element *element,
 				*((char **) value) = property->value;
 				found = TRUE;
 				break;
+			case DBUS_TYPE_BYTE:
+				memcpy(value, property->value, 1);
+				found = TRUE;
+				break;
 			}
 			break;
 		}
@@ -1609,6 +1044,10 @@ gboolean connman_element_get_static_property(struct connman_element *element,
 
 	__connman_element_unlock(element);
 
+	if (found == FALSE && element->parent != NULL)
+		return connman_element_get_static_property(element->parent,
+								name, value);
+
 	return found;
 }
 
@@ -1673,67 +1112,6 @@ gboolean connman_element_match_static_property(struct connman_element *element,
 	return result;
 }
 
-static void append_devices(DBusMessageIter *entry)
-{
-	DBusMessageIter value, iter;
-	const char *key = "Devices";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_DEVICE, &iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
-}
-
-static void emit_devices_signal(DBusConnection *conn)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry;
-
-	DBG("conn %p", conn);
-
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	append_devices(&entry);
-
-	g_dbus_send_message(conn, signal);
-}
-
-static void emit_networks_signal(DBusConnection *conn,
-					struct connman_element *device)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry;
-
-	DBG("conn %p", conn);
-
-	if (device == NULL)
-		return;
-
-	signal = dbus_message_new_signal(device->path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	append_networks(device, &entry);
-
-	g_dbus_send_message(conn, signal);
-}
-
 static void append_connections(DBusMessageIter *entry)
 {
 	DBusMessageIter value, iter;
@@ -1804,17 +1182,26 @@ static void emit_state_change(DBusConnection *conn, const char *state)
 	g_dbus_send_message(conn, signal);
 }
 
-static void set_signal_strength(struct connman_element *connection)
+static void probe_element(struct connman_element *element)
 {
-	struct connman_element *element = connection;
+	GSList *list;
 
-	while (element != NULL) {
-		if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
-			connection->strength = element->strength;
+	DBG("element %p name %s", element, element->name);
+
+	for (list = driver_list; list; list = list->next) {
+		struct connman_driver *driver = list->data;
+
+		if (match_driver(element, driver) == FALSE)
+			continue;
+
+		DBG("driver %p name %s", driver, driver->name);
+
+		if (driver->probe(element) == 0) {
+			__connman_element_lock(element);
+			element->driver = driver;
+			__connman_element_unlock(element);
 			break;
 		}
-
-		element = element->parent;
 	}
 }
 
@@ -1822,7 +1209,6 @@ static void register_element(gpointer data, gpointer user_data)
 {
 	struct connman_element *element = data;
 	const gchar *basepath;
-	GSList *list;
 	GNode *node;
 
 	__connman_element_lock(element);
@@ -1831,9 +1217,6 @@ static void register_element(gpointer data, gpointer user_data)
 		node = g_node_find(element_root, G_PRE_ORDER,
 					G_TRAVERSE_ALL, element->parent);
 		basepath = element->parent->path;
-
-		if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_UNKNOWN)
-			element->subtype = element->parent->subtype;
 	} else {
 		element->parent = element_root->data;
 
@@ -1849,66 +1232,19 @@ static void register_element(gpointer data, gpointer user_data)
 
 	DBG("element %p path %s", element, element->path);
 
-	__connman_element_load(element);
-
 	g_node_append_data(node, element);
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
-			element->subtype != CONNMAN_ELEMENT_SUBTYPE_NETWORK) {
-		if (g_dbus_register_interface(connection, element->path,
-					CONNMAN_DEVICE_INTERFACE,
-					device_methods, element_signals,
-					NULL, element, NULL) == FALSE)
-			connman_error("Failed to register %s device",
-								element->path);
-		else
-			emit_devices_signal(connection);
-	}
-
-	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
-		if (g_dbus_register_interface(connection, element->path,
-					CONNMAN_NETWORK_INTERFACE,
-					network_methods, element_signals,
-					NULL, element, NULL) == FALSE)
-			connman_error("Failed to register %s network",
-								element->path);
-		else
-			emit_networks_signal(connection, element->parent);
-	}
-
 	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
-		if (g_dbus_register_interface(connection, element->path,
-					CONNMAN_CONNECTION_INTERFACE,
-					connection_methods, element_signals,
-					NULL, element, NULL) == FALSE)
-			connman_error("Failed to register %s connection",
-								element->path);
-		else {
-			set_signal_strength(element);
-			emit_connections_signal(connection);
-			emit_state_change(connection, "online");
-		}
+		emit_connections_signal(connection);
+		emit_state_change(connection, "online");
 	}
 
-	__connman_element_store(element);
+	emit_element_signal(connection, "ElementAdded", element);
 
-	for (list = driver_list; list; list = list->next) {
-		struct connman_driver *driver = list->data;
+	if (started == FALSE)
+		return;
 
-		if (match_driver(element, driver) == FALSE)
-			continue;
-
-		DBG("driver %p name %s", driver, driver->name);
-
-		if (driver->probe(element) == 0) {
-			__connman_element_lock(element);
-			element->driver = driver;
-			__connman_element_unlock(element);
-
-			enable_element(element);
-			break;
-		}
-	}
+	probe_element(element);
 }
 
 /**
@@ -1929,8 +1265,7 @@ int connman_element_register(struct connman_element *element,
 	if (element->devname == NULL)
 		element->devname = g_strdup(element->name);
 
-	if (device_filter && element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
-			element->subtype != CONNMAN_ELEMENT_SUBTYPE_NETWORK) {
+	if (device_filter && element->type == CONNMAN_ELEMENT_TYPE_DEVICE) {
 		if (g_pattern_match_simple(device_filter,
 						element->devname) == FALSE) {
 			DBG("ignoring %s [%s] device", element->name,
@@ -1952,6 +1287,9 @@ int connman_element_register(struct connman_element *element,
 		}
 	}
 
+	if (element->type == CONNMAN_ELEMENT_TYPE_DHCP)
+		element->ipv4.method = CONNMAN_IPV4_METHOD_DHCP;
+
 	element->parent = parent;
 
 	__connman_element_unlock(element);
@@ -1971,9 +1309,10 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 	if (element == root)
 		return FALSE;
 
-	if (element->driver) {
-		disable_element(element);
+	if (node != NULL)
+		g_node_unlink(node);
 
+	if (element->driver) {
 		if (element->driver->remove)
 			element->driver->remove(element);
 
@@ -1982,35 +1321,17 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 		__connman_element_unlock(element);
 	}
 
-	if (node != NULL) {
-		g_node_unlink(node);
+	if (node != NULL)
 		g_node_destroy(node);
-	}
 
 	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
 		if (__connman_element_count(NULL,
 					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
 			emit_state_change(connection, "offline");
 		emit_connections_signal(connection);
-
-		g_dbus_unregister_interface(connection, element->path,
-						CONNMAN_CONNECTION_INTERFACE);
 	}
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
-		emit_networks_signal(connection, element->parent);
-
-		g_dbus_unregister_interface(connection, element->path,
-						CONNMAN_NETWORK_INTERFACE);
-	}
-
-	if (element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
-			element->subtype != CONNMAN_ELEMENT_SUBTYPE_NETWORK) {
-		emit_devices_signal(connection);
-
-		g_dbus_unregister_interface(connection, element->path,
-						CONNMAN_DEVICE_INTERFACE);
-	}
+	emit_element_signal(connection, "ElementRemoved", element);
 
 	connman_element_unref(element);
 
@@ -2046,21 +1367,13 @@ void connman_element_unregister_children(struct connman_element *element)
 static gboolean update_element(GNode *node, gpointer user_data)
 {
 	struct connman_element *element = node->data;
-	struct connman_element *root = user_data;
 
 	DBG("element %p name %s", element, element->name);
 
 	if (element->driver && element->driver->update)
 		element->driver->update(element);
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION &&
-				root->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
-		if (element->strength != root->strength) {
-			element->strength = root->strength;
-			emit_property_changed(connection, element, "Strength",
-					DBUS_TYPE_BYTE, &element->strength);
-		}
-	}
+	emit_element_signal(connection, "ElementUpdated", element);
 
 	return FALSE;
 }
@@ -2086,8 +1399,6 @@ int connman_element_set_enabled(struct connman_element *element,
 
 	element->enabled = enabled;
 
-	emit_enabled_signal(connection, element);
-
 	return 0;
 }
 
@@ -2112,11 +1423,52 @@ int __connman_element_init(DBusConnection *conn, const char *device)
 
 	element_root = g_node_new(element);
 
+	__connman_network_init();
 	__connman_device_init();
 
 	return 0;
 }
 
+static gboolean probe_node(GNode *node, gpointer data)
+{
+	struct connman_element *element = node->data;
+
+	DBG("element %p name %s", element, element->name);
+
+	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
+		return FALSE;
+
+	if (element->driver)
+		return FALSE;
+
+	probe_element(element);
+
+	return FALSE;
+}
+
+void __connman_element_start(void)
+{
+	DBG("");
+
+	g_node_traverse(element_root, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+							probe_node, NULL);
+
+	started = TRUE;
+
+	__connman_connection_init();
+	__connman_ipv4_init();
+	__connman_detect_init();
+}
+
+void __connman_element_stop(void)
+{
+	DBG("");
+
+	__connman_detect_cleanup();
+	__connman_ipv4_cleanup();
+	__connman_connection_cleanup();
+}
+
 static gboolean free_driver(GNode *node, gpointer data)
 {
 	struct connman_element *element = node->data;
@@ -2124,8 +1476,6 @@ static gboolean free_driver(GNode *node, gpointer data)
 	DBG("element %p name %s", element, element->name);
 
 	if (element->driver) {
-		disable_element(element);
-
 		if (element->driver->remove)
 			element->driver->remove(element);
 
@@ -2154,6 +1504,7 @@ void __connman_element_cleanup(void)
 	DBG("");
 
 	__connman_device_cleanup();
+	__connman_network_cleanup();
 
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 							free_driver, NULL);
